@@ -16,8 +16,7 @@
  ***************************************************************************/
 
 // QT includes
-#include <qstringlist.h>
-#include <qpushbutton.h>
+#include <QtGui/QPushButton>
 
 // KDE includes
 #include <kdebug.h>
@@ -70,7 +69,7 @@ void FX_SetupImpl::slotRemove()
 
 void FX_SetupImpl::slotRemoveAll()
 {
-  kdDebug() << "FX_SetupImpl::slotRemoveAll()" << endl;
+  kDebug() << "FX_SetupImpl::slotRemoveAll()";
   
   // move all from Active-List to Available-List
   for(uint i=0; i<listBox_Active->count(); i++){
@@ -83,13 +82,13 @@ void FX_SetupImpl::slotRemoveAll()
 
 void FX_SetupImpl::slotReload()
 {
-    kdDebug() << "FX_SetupImpl::slotReload()" << endl;
+    kDebug() << "FX_SetupImpl::slotReload()";
     
     Init(pluginlist);
 }
 
 
-void FX_SetupImpl::slotConfigureEffect(QListBoxItem *item)
+void FX_SetupImpl::slotConfigureEffect(Q3ListBoxItem *item)
 {
   m_fxpluginhandler->showEffectGUI(item->text());
 }
@@ -98,7 +97,7 @@ void FX_SetupImpl::slotConfigureEffect(QListBoxItem *item)
 void FX_SetupImpl::Init(QStringList c_avail)
 {
   m_config->setGroup("Effect Stack Configuration");
-  QStringList conf_active = m_config->readListEntry("Activated");
+  QStringList conf_active = m_config->readEntry("Activated",QStringList());
   QStringList c_active;
   QStringList::Iterator sit, it;
   
@@ -144,14 +143,11 @@ void FX_SetupImpl::slotSaveWasClicked()
 
   // Read ListBox Activated FX
   slist.clear();
-  QString t;
   for(uint i=0; i<listBox_Active->count(); i++){
-    t = listBox_Active->text( (int)i );
-    slist.append( t );
+    slist.append( listBox_Active->text( (int)i) );
   }
   // Write StringList
   m_config->writeEntry("Activated", slist);
-  m_config->sync();
   slist.clear();
 }
 
