@@ -1,7 +1,7 @@
 /***************************************************** vim:set ts=4 sw=4 sts=4:
   KControl module for KTTSD configuration and Job Management
   -------------------
-  Copyright : (C) 2002-2003 by José Pablo Ezequiel "Pupeno" Fernández
+  Copyright : (C) 2002-2003 by José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
   Copyright : (C) 2004-2005 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
@@ -20,6 +20,10 @@
 // synthesizer name.  The latter is a translated name, for example, "Festival Interactivo",
 // while the former is alway an English name, example "Festival Interactive".
 
+// KCMKttsMgr includes.
+#include "kcmkttsmgr.h"
+#include "kcmkttsmgr.moc"
+
 // C++ includes.
 #include <math.h>
 
@@ -66,10 +70,6 @@
 #include "selectlanguagedlg.h"
 #include "utils.h"
 
-// KCMKttsMgr includes.
-#include "kcmkttsmgr.h"
-#include "kcmkttsmgr.moc"
-
 // Some constants.
 // Defaults set when clicking Defaults button.
 const bool autostartMgrCheckBoxValue = true;
@@ -1715,7 +1715,7 @@ void KCMKttsMgr::kttsdStarted()
         if (m_jobMgrPart)
         {
             // Add the Job Manager part as a new tab.
-            mainTab->addTab(m_jobMgrPart->widget(), i18n("&Jobs"));
+            mainTab->addTab(m_jobMgrPart->widget(), i18n("Jobs"));
             kttsdLoaded = true;
         }
         else
@@ -2002,13 +2002,13 @@ void KCMKttsMgr::keepAudioCheckBox_toggled(bool checked)
 // with alpha = 1000/(log(200)-log(50))
 
 int KCMKttsMgr::percentToSlider(int percentValue) {
-    double alpha = 1000 / (log(200) - log(50));
-    return (int)floor (0.5 + alpha * (log(percentValue)-log(50)));
+    double alpha = 1000 / (log(200.0) - log(50.0));
+    return (int)floor (0.5 + alpha * (log((float)percentValue)-log(50.0)));
 }
 
 int KCMKttsMgr::sliderToPercent(int sliderValue) {
-    double alpha = 1000 / (log(200) - log(50));
-    return (int)floor(0.5 + exp (sliderValue/alpha + log(50)));
+    double alpha = 1000 / (log(200.0) - log(50.0));
+    return (int)floor(0.5 + exp (sliderValue/alpha + log(50.0)));
 }
 
 void KCMKttsMgr::timeBox_valueChanged(int percentValue) {
