@@ -1,8 +1,8 @@
 /***************************************************** vim:set ts=4 sw=4 sts=4:
   KControl module for KTTSD configuration and job management
   -------------------
-  Copyright : (C) 2002-2003 by José Pablo Ezequiel "Pupeno" Fernández
-  Copyright : (C) 2004 by Gary Cramblitt <garycramblitt@comcast.net>
+  Copyright : (C) 2002-2003 by José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
+  Copyright : (C) 2004-2005 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
   Current Maintainer: 2004 by Gary Cramblitt <garycramblitt@comcast.net>
@@ -16,48 +16,100 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifndef KCMKTTSMGR_H
-#define KCMKTTSMGR_H
-
-#include "config.h"
-
-// Qt includes.
-#include <qmap.h>
-#include <qlistview.h>
-
-// KDE includes.
-#include <kcmodule.h>
-#include <ktrader.h>
-#include <kdebug.h>
-#include <kparts/part.h>
-
-// KTTS includes.
-#include "addtalker.h"
-#include "kcmkttsmgrwidget.h"
-#include "kspeech_stub.h"
-#include "kspeechsink.h"
-
-class PlugInConf;
-class KttsFilterConf;
-class KListViewItem;
-class KAboutData;
-class KConfig;
-class QPopupMenu;
-
 /**
 * @author José Pablo Ezequiel "Pupeno" Fernández
 * @author Gary Cramblitt
 */
 
+#ifndef KCMKTTSMGR_H
+#define KCMKTTSMGR_H
+
+// Qt includes.
+#include <QtCore/QMap>
+#include <QtCore/QAbstractListModel>
+#include <QtCore/QModelIndex>
+
+// KDE includes.
+#include <kcmodule.h>
+#include <kdebug.h>
+#include <kparts/part.h>
+
+// KTTS includes.
+#include "talkercode.h"
+#include "talkerlistmodel.h"
+#include "addtalker.h"
+#include "ui_kcmkttsmgrwidget.h"
+#include "kspeechinterface.h"
+
+class PlugInConf;
+class KttsFilterConf;
+class QTreeWidget;
+class QTreeWidgetItem;
+class KAboutData;
+class KConfig;
+class QAction;
+
+class FilterItem
+{
+public:
+    QString id;
+    QString userFilterName;
+    QString plugInName;
+    QString desktopEntryName;
+    bool enabled;
+    bool multiInstance;
+};
+
+typedef QList<FilterItem> FilterList;
+
+class FilterListModel : public QAbstractListModel
+{
+    Q_OBJECT
+public:
+    explicit FilterListModel(FilterList filters = FilterList(), QObject *parent = 0);
+
+    FilterList datastore() { return m_filters; }
+    void setDatastore(FilterList filters = FilterList()) { m_filters = filters; }
+    int rowCount(const QModelIndex &parent = QModelIndex()) const;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const;
+    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
+    QModelIndex parent(const QModelIndex & index ) const;
+    QVariant data(const QModelIndex &index, int role) const;
+    QVariant headerData(int section, Qt::Orientation orientation,
+        int role = Qt::DisplayRole) const;
+    Qt::ItemFlags flags(const QModelIndex &index) const;
+    bool removeRow(int row, const QModelIndex & parent = QModelIndex());
+
+    FilterItem getRow(int row) const;
+    bool appendRow(FilterItem& filter);
+    bool updateRow(int row, FilterItem& filter);
+    bool swap(int i, int j);
+    void clear();
+protected:
+    FilterList m_filters;
+};
+
+class SbdFilterListModel : public FilterListModel
+{
+    Q_OBJECT
+public:
+    explicit SbdFilterListModel(FilterList filters = FilterList(), QObject *parent = 0);
+
+    int columnCount(const QModelIndex &parent = QModelIndex()) const;
+    QVariant data(const QModelIndex &index, int role) const;
+    QVariant headerData(int section, Qt::Orientation orientation,
+        int role = Qt::DisplayRole) const;
+    Qt::ItemFlags flags(const QModelIndex &index) const;
+};
+
 class KCMKttsMgr :
     public KCModule,
-    public KSpeech_stub,
-    virtual public KSpeechSink
+    private Ui::KCMKttsMgrWidget
 {
     Q_OBJECT
 
     public:
-        KCMKttsMgr(QWidget *parent, const char *name, const QStringList &);
+        KCMKttsMgr(QWidget *parent, const QVariantList &);
 
         ~KCMKttsMgr();
 
@@ -97,14 +149,6 @@ class KCMKttsMgr :
         */
         static void init();
 
-        /**
-        * The control center calls this function to decide which buttons should
-        * be displayed. For example, it does not make sense to display an "Apply" 
-        * button for one of the information modules. The value returned can be set by 
-        * modules using setButtons.
-        */
-        int buttons();
-
         /**
         * This function returns the small quickhelp.
         * That is displayed in the sidebar in the KControl
@@ -124,17 +168,13 @@ class KCMKttsMgr :
         {
             if (!m_suppressConfigChanged)
             {
-                // kdDebug() << "KCMKttsMgr::configChanged: Running"<< endl;
+                // kDebug() << "KCMKttsMgr::configChanged: Running";
                 m_changed = true;
                 emit changed(true);
             }
-        };
-        /**
-        * This slot is called whenever user checks/unchecks item in Filters list.
-        */
-        void slotFiltersList_stateChanged();
+        }
 
-    protected:
+    protected slots:
         /** DCOP Methods connected to DCOP Signals emitted by KTTSD. */
         /** Most of these are not used */
 
@@ -154,59 +194,10 @@ class KCMKttsMgr :
         {
             wpGeneral = 0,          // General tab.
             wpTalkers = 1,          // Talkers tab.
-            wpNotify = 2,           // Notify tab.
-            wpFilters = 3,          // Filters tab.
-            wpInterruption = 4,     // Interruption tab.
-            wpAudio = 5,            // Audio tab.
-            wpJobs = 6              // Jobs tab.
-        };
-
-        enum NotifyListViewColumn
-        {
-            nlvcEventSrcName = 0,
-            nlvcEventName = 0,
-            nlvcActionName = 1,
-            nlvcTalkerName = 2,
-            nlvcEventSrc = 3,      // hidden
-            nlvcEvent = 4,         // hidden
-            nlvcAction = 5,        // hidden
-            nlvcTalker = 6         // hidden
-        };
-
-        enum TalkerListViewColumn
-        {
-            tlvcTalkerID,
-            tlvcLanguage,
-            tlvcSynthName,
-            tlvcVoice,
-            tlvcGender,
-            tlvcVolume,
-            tlvcRate
-        };
-
-        enum FilterListViewColumn
-        {
-            flvcUserName,           // Name of filter as set by user and displayed.
-            flvcFilterID,           // Internal ID assigned to the filter (hidden).
-            flvcPlugInName,         // Name of the filter plugin (from .desktop file, hidden).
-            flvcMultiInstance       // True if multiple instances of this plugin are possible. (hidden)
-        };
-
-        enum SbdListViewColumn
-        {
-            slvcUserName,           // Name of filter as set by user and displayed.
-            slvcFilterID,           // Internal ID assigned to the filter (hidden).
-            slvcPlugInName,         // Name of the filter plugin (from .desktop file, hidden).
-            slvcMultiInstance       // True if multiple instances of this plugin are possible. (hidden)
-        };
-
-        enum SbdButtonIDs
-        {
-            sbdBtnEdit = 1,
-            sbdBtnUp = 2,
-            sbdBtnDown = 3,
-            sbdBtnAdd = 4,
-            sbdBtnRemove = 5
+            wpFilters = 2,          // Filters tab.
+            wpInterruption = 3,     // Interruption tab.
+            wpAudio = 4,            // Audio tab.
+            wpJobs = 5              // Jobs tab.
         };
 
         /**
@@ -228,13 +219,6 @@ class KCMKttsMgr :
         */
         QString defaultTalkerCode(const QString &languageCode, const QString &plugInName);
 
-        /**
-        * Given an item in the talker listview and a talker code, sets the columns of the item.
-        * @param talkerItem       QListViewItem.
-        * @param talkerCode       Talker Code.
-        */
-        void updateTalkerItem(QListViewItem* talkerItem, const QString &talkerCode);
-
         /**
         * Loads the configuration plugin for a named Talker plugin.
         * @param name             DesktopEntryName of the Synthesizer.
@@ -270,12 +254,6 @@ class KCMKttsMgr :
         */
         void removeFilter( bool sbd );
 
-        /**
-        * Move an item in a KListView up or down.
-        */
-        void lowerItemPriority( KListView* lView );
-        void higherItemPriority( KListView* lView );
-
         /**
         * Count number of configured Filters with the specified plugin name.
         */
@@ -285,7 +263,7 @@ class KCMKttsMgr :
          * Uses KTrader to convert a translated Filter Plugin Name to DesktopEntryName.
          * @param name                   The translated plugin name.  From Name= line in .desktop file.
          * @return                       DesktopEntryName.  The name of the .desktop file (less .desktop).
-         *                               QString::null if not found.
+         *                               QString() if not found.
          */
         QString FilterNameToDesktopEntryName(const QString& name);
 
@@ -296,31 +274,23 @@ class KCMKttsMgr :
          */
         QString FilterDesktopEntryNameToName(const QString& desktopEntryName);
 
-        /**
-         * Loads notify events from a file.  Clearing listview if clear is True.
-         */
-        QString loadNotifyEventsFromFile( const QString& filename, bool clear);
 
         /**
-         * Saves notify events to a file.
-         */
-        QString saveNotifyEventsToFile(const QString& filename);
-
-        /**
-         * Adds an item to the notify listview.
-         * message is only needed if action = nactSpeakCustom.
-         */
-        QListViewItem* addNotifyItem(
-            const QString& eventSrc,
-            const QString& event,
-            int action,
-            const QString& message,
-            TalkerCode& talkerCode);
-
-        /**
-        * Main widget
+        * A convenience method that finds an item in a TreeViewWidget, assuming there is at most
+        * one occurrence of the item.  Returns 0 if not found.
+        * @param tw                     The TreeViewWidget to search.
+        * @param sought                 The string sought.
+        * @param col                    Column of the TreeViewWidget to search.
+        * @return                       The item of the TreeViewWidget found or null if not found.
+        *
+        * An exact match is performed.
         */
-        KCMKttsMgrWidget *m_kttsmgrw;
+        QTreeWidgetItem* findTreeWidgetItem(QTreeWidget* tw, const QString& sought, int col);
+        
+        /**
+        * DBUS KSpeech Interface.
+        */
+        org::kde::KSpeech* m_kspeech;
 
         /**
         * Object holding all the configuration
@@ -335,12 +305,16 @@ class KCMKttsMgr :
         /**
         * Plugin configuration dialog.
         */
-        KDialogBase* m_configDlg;
+        KDialog* m_configDlg;
 
         /**
-        * Sentence Boundary Detector button popup menu.
+        * Sentence Boundary Detector button popup menu actions.
         */
-        QPopupMenu* m_sbdPopmenu;
+        QAction* m_sbdBtnEdit;
+        QAction* m_sbdBtnUp;
+        QAction* m_sbdBtnDown;
+        QAction* m_sbdBtnAdd;
+        QAction* m_sbdBtnRemove;
 
         /**
         * Talker(synth) Plugin currently loaded into configuration dialog.
@@ -352,11 +326,22 @@ class KCMKttsMgr :
         */
         KttsFilterConf *m_loadedFilterPlugIn;
 
+        /**
+        * Model containing list of Talker Codes.
+        */
+        TalkerListModel m_talkerListModel;
+
         /**
         * Last talker ID.  Used to generate a new ID.
         */
         int m_lastTalkerID;
 
+        /**
+        * Models containing normal and SBD filters.
+        */
+        FilterListModel m_filterListModel;
+        SbdFilterListModel m_sbdFilterListModel;
+
         /**
         * Last filter ID.  Used to generate a new ID.
         */
@@ -388,44 +373,7 @@ class KCMKttsMgr :
         */
         SynthToLangMap m_synthToLangMap;
 
-        /**
-        * Default Talker Code for notifications.
-        */
-        QString m_defaultNotifyTalkerCode;
-
     private slots:
-        /**
-        * Add a talker/filter.
-        * This is a wrapper function that takes the parameters for the real talker from the
-        * widgets to later call it.
-        */
-        void slot_addTalker();
-        void slot_addNormalFilter();
-        void slot_addSbdFilter();
-
-        /**
-        * Remove talker/filter.
-        * This is a wrapper function that takes the parameters for the real removeTalker from the
-        * widgets to later call it.
-        */
-        void slot_removeTalker();
-        void slot_removeNormalFilter();
-        void slot_removeSbdFilter();
-
-        /**
-        * This slot is called whenever user clicks the higher*Priority button (up).
-        */
-        void slot_higherTalkerPriority();
-        void slot_higherNormalFilterPriority();
-        void slot_higherSbdFilterPriority();
-
-        /**
-        * This slot is called whenever user clicks the lower*Priority button (down).
-        */
-        void slot_lowerTalkerPriority();
-        void slot_lowerNormalFilterPriority();
-        void slot_lowerSbdFilterPriority();
-
         /**
         * Update the status of the Talker/Filter buttons.
         */
@@ -433,38 +381,6 @@ class KCMKttsMgr :
         void updateFilterButtons();
         void updateSbdButtons();
 
-        /**
-        * This signal is emitted whenever user checks/unchecks the Enable TTS System check box.
-        */
-        void enableKttsdToggled(bool checked);
-
-        /**
-        * This signal is emitted whenever user checks/unchecks the GStreamer radio button.
-        */
-        void slotGstreamerRadioButton_toggled(bool state);
-
-        /**
-        * This signal is emitted whenever user checks/unchecks the ALSA radio button.
-        */
-        void slotAlsaRadioButton_toggled(bool state);
-
-        /**
-        * This is emitted whenever user activates the ALSA pcm combobox.
-        */
-        void slotPcmComboBox_activated();
-
-        /**
-        * This signal is emitted whenever user checks/unchecks the aKode radio button.
-        */
-        void slotAkodeRadioButton_toggled(bool state);
-
-        /**
-        * User has requested to display the Talker/Filter Configuration Dialog.
-        */
-        void slot_configureTalker();
-        void slot_configureNormalFilter();
-        void slot_configureSbdFilter();
-
         /**
         * Slots for the Talker/Filter Configuration dialogs.
         */
@@ -476,31 +392,53 @@ class KCMKttsMgr :
         void slotConfigFilterDlg_CancelClicked();
 
         /**
-        * Slots for Speed setting.
+        * General tab slots.
+        */
+        void slotEnableKttsd_toggled(bool checked);
+        void slotAutoStartMgrCheckBox_toggled(bool checked);
+
+        /**
+        * Talker tab slots.
+        */
+        void slotAddTalkerButton_clicked();
+        void slotRemoveTalkerButton_clicked();
+        void slotHigherTalkerPriorityButton_clicked();
+        void slotLowerTalkerPriorityButton_clicked();
+        void slotConfigureTalkerButton_clicked();
+
+
+        /**
+        * Filters tab slots.
+        */
+        void slotFilterListView_clicked(const QModelIndex & index);
+        void slotAddNormalFilterButton_clicked();
+        void slotAddSbdFilterButton_clicked();
+        void slotRemoveNormalFilterButton_clicked();
+        void slotRemoveSbdFilterButton_clicked();
+        void slotHigherNormalFilterPriorityButton_clicked();
+        void slotHigherSbdFilterPriorityButton_clicked();
+        void slotLowerNormalFilterPriorityButton_clicked();
+        void slotLowerSbdFilterPriorityButton_clicked();
+        void slotConfigureNormalFilterButton_clicked();
+        void slotConfigureSbdFilterButton_clicked();
+
+        /**
+        * Interruption tab slots.
+        */
+        void slotTextPreMsgCheck_toggled(bool checked);
+        void slotTextPreSndCheck_toggled(bool checked);
+        void slotTextPostMsgCheck_toggled(bool checked);
+        void slotTextPostSndCheck_toggled(bool checked);
+
+        /**
+        * Audio tab slots.
         */
         void timeBox_valueChanged(int percentValue);
         void timeSlider_valueChanged(int sliderValue);
-
-        /**
-        * Keep Audio CheckBox slot.
-        */
         void keepAudioCheckBox_toggled(bool checked);
-
-        /**
-        * Notify tab slots.
-        */
-        void slotNotifyEnableCheckBox_toggled(bool checked);
-        void slotNotifyAddButton_clicked();
-        void slotNotifyRemoveButton_clicked();
-        void slotNotifyClearButton_clicked();
-        void slotNotifyLoadButton_clicked();
-        void slotNotifySaveButton_clicked();
-        void slotNotifyListView_selectionChanged();
-        void slotNotifyPresentComboBox_activated(int index);
-        void slotNotifyActionComboBox_activated(int index);
-        void slotNotifyTestButton_clicked();
-        void slotNotifyMsgLineEdit_textChanged(const QString& text);
-        void slotNotifyTalkerButton_clicked();
+        void slotPhononRadioButton_toggled(bool state);
+        void slotAlsaRadioButton_toggled(bool state);
+        void slotPcmComboBox_activated();
 
         /**
         * Other slots.
@@ -508,23 +446,4 @@ class KCMKttsMgr :
         void slotTabChanged();
 };
 
-/// This is a small helper class to detect when user checks/unchecks a Filter in Filters tab
-/// and emit changed() signal.
-class KttsCheckListItem : public QCheckListItem
-{
-    public:
-        KttsCheckListItem( QListView *parent,
-            const QString &text, Type tt = RadioButtonController,
-            KCMKttsMgr* kcmkttsmgr = 0);
-        KttsCheckListItem( QListView *parent, QListViewItem *after,
-            const QString &text, Type tt = RadioButtonController,
-            KCMKttsMgr* kcmkttsmgr = 0);
-
-    protected:
-        virtual void stateChange(bool);
-
-    private:
-        KCMKttsMgr* m_kcmkttsmgr;
-};
-
 #endif
