@@ -1,7 +1,8 @@
 /***************************************************** vim:set ts=4 sw=4 sts=4:
   Main speaking functions for the Command Plug in
   -------------------
-  Copyright : (C) 2002 by Gunnar Schmi Dt and 2004 by Gary Cramblitt
+  Copyright : (C) 2002 by Gunnar Schmi Dt <kmouth@schmi-dt.de>
+  Copyright : (C) 2004 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: Gunnar Schmi Dt <kmouth@schmi-dt.de>
   Current Maintainer: Gary Cramblitt <garycramblitt@comcast.net>
@@ -11,38 +12,38 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; version 2 of the License.               *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
 
-// Qt includes.
-#include <qfile.h>
-#include <qstring.h>
-#include <qvaluelist.h>
-#include <qstringlist.h>
-#include <qregexp.h>
-#include <qtextcodec.h>
-#include <qvaluestack.h>
-
-// KDE includes.
-#include <kdebug.h>
-#include <kconfig.h>
-#include <kprocess.h>
-#include <ktempfile.h>
-#include <kstandarddirs.h>
-
-// KTTS includes.
-#include <pluginproc.h>
-
 // Command Plugin includes.
 #include "commandproc.h"
 #include "commandproc.moc"
 
+// Qt includes.
+#include <QtCore/QFile>
+#include <QtCore/QRegExp>
+#include <QtCore/QTextCodec>
+#include <QtCore/QStack>
+#include <QtCore/QTextStream>
+
+// KDE includes.
+#include <kdebug.h>
+#include <kconfig.h>
+#include <kconfiggroup.h>
+#include <k3process.h>
+#include <ktemporaryfile.h>
+#include <kstandarddirs.h>
+
+// KTTS includes.
+#include <pluginproc.h>
+
 /** Constructor */
-CommandProc::CommandProc( QObject* parent, const char* name, const QStringList& /*args*/) : 
-    PlugInProc( parent, name )
+CommandProc::CommandProc( QObject* parent, const QStringList& /*args*/) : 
+    PlugInProc( parent, "commandproc" )
 {
-    kdDebug() << "CommandProc::CommandProc: Running" << endl;
+    kDebug() << "CommandProc::CommandProc: Running";
     m_commandProc = 0;
     m_state = psIdle;
     m_stdin = true;
@@ -53,7 +54,7 @@ CommandProc::CommandProc( QObject* parent, const char* name, const QStringList&
 /** Destructor */
 CommandProc::~CommandProc()
 {
-    kdDebug() << "CommandProc::~CommandProc: Running" << endl;
+    kDebug() << "CommandProc::~CommandProc: Running";
     if (m_commandProc)
     {
         if (m_commandProc->isRunning()) m_commandProc->kill();
@@ -64,20 +65,20 @@ CommandProc::~CommandProc()
 }
 
 /** Initialize */
-bool CommandProc::init(KConfig *config, const QString &configGroup){
-    kdDebug() << "CommandProc::init: Initializing plug in: Command " << endl;
+bool CommandProc::init(KConfig *c, const QString &configGroup){
+    kDebug() << "CommandProc::init: Initializing plug in: Command ";
 
-    config->setGroup(configGroup);
-    m_ttsCommand = config->readEntry("Command", "cat -");
-    m_stdin = config->readBoolEntry("StdIn", true);
-    m_language = config->readEntry("LanguageCode", "en");
+    KConfigGroup config(c, configGroup);
+    m_ttsCommand = config.readEntry("Command", "cat -");
+    m_stdin = config.readEntry("StdIn", true);
+    m_language = config.readEntry("LanguageCode", "en");
 
     // Support separate synthesis if the TTS command contains %w macro.
     m_supportsSynth = (m_ttsCommand.contains("%w"));
 
-    QString codecString = config->readEntry("Codec", "Local");
+    QString codecString = config.readEntry("Codec", "Local");
     m_codec = codecNameToCodec(codecString);
-    kdDebug() << "CommandProc::init: Initialized with command: " << m_ttsCommand << " codec: " << codecString << endl;
+    kDebug() << "CommandProc::init: Initialized with command: " << m_ttsCommand << " codec: " << codecString;
     return true;
 }
 
@@ -89,7 +90,7 @@ bool CommandProc::init(KConfig *config, const QString &configGroup){
 */
 void CommandProc::sayText(const QString &text)
 {
-    synth(text, QString::null,
+    synth(text, QString(),
         m_ttsCommand, m_stdin, m_codec, m_language);
 }
 
@@ -115,7 +116,7 @@ void CommandProc::synthText(const QString& text, const QString& suggestedFilenam
 * @param suggestedFilename       If not Null, synthesize only to this filename, otherwise
 *                                synthesize and audibilize the text.
 * @param userCmd                 The program that shall be executed for speaking
-* @param stdIn                   True if the program shall recieve its data via standard input
+* @param stdIn                   True if the program shall receive its data via standard input
 * @param codec                   The QTextCodec if encoding==UseCodec
 * @param language                The language code (used for the %l macro)
 */
@@ -127,39 +128,41 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
         if (m_commandProc->isRunning()) m_commandProc->kill();
         delete m_commandProc;
         m_commandProc = 0;
-        m_synthFilename = QString::null;
+        m_synthFilename.clear();
         if (!m_textFilename.isNull()) QFile::remove(m_textFilename);
-        m_textFilename = QString::null;
+        m_textFilename.clear();
     }
     QString command = userCmd;
-    QString text = inputText.stripWhiteSpace();
+    QString text = inputText.trimmed();
     if (text.isEmpty()) return;
-     // 1. prepare the text:
-     // 1.a) encode the text
-    text += "\n";
-    QCString encodedText;
-    if (codec)
-        encodedText = codec->fromUnicode(text);
-    else
-        encodedText = text.latin1();  // Should not happen, but just in case.
+    // 1. prepare the text:
+    // 1.a) encode the text
+    QByteArray encText;
+    QTextStream ts (encText, QIODevice::WriteOnly);
+    ts.setCodec(codec);
+    ts << text;
+    ts << endl; // Some synths need this, eg. flite.
 
     // 1.b) quote the text as one parameter
-    QString escText = KShellProcess::quote(text);
+    QString escText = K3ShellProcess::quote(text);
 
     // 1.c) create a temporary file for the text, if %f macro is used.
     if (command.contains("%f"))
     {
-        KTempFile tempFile(locateLocal("tmp", "commandplugin-"), ".txt");
-        QTextStream* fs = tempFile.textStream();
-        fs->setCodec(codec);
-        *fs << text;
-        *fs << endl;
-        m_textFilename = tempFile.file()->name();
-        tempFile.close();
-    } else m_textFilename = QString::null;
+        KTemporaryFile tempFile;
+        tempFile.setPrefix("commandplugin-");
+        tempFile.setSuffix(".txt");
+        tempFile.setAutoRemove(false);
+        tempFile.open();
+        QTextStream fs (&tempFile);
+        fs.setCodec(codec);
+        fs << text;
+        fs << endl;
+        m_textFilename = tempFile.fileName();
+    } else m_textFilename.clear();
 
     // 2. replace variables with values
-    QValueStack<bool> stack;
+    QStack<bool> stack;
     bool issinglequote=false;
     bool isdoublequote=false;
     int noreplace=0;
@@ -167,11 +170,11 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
     QRegExp re_singlequote("('|%%|%t|%f|%l|%w)");
     QRegExp re_doublequote("(\"|\\\\|`|\\$\\(|\\$\\{|%%|%t|%f|%l|%w)");
 
-    for	( int i = re_noquote.search(command);
+    for	( int i = re_noquote.indexIn(command);
         i != -1;
-        i = (issinglequote?re_singlequote.search(command,i)
-            :isdoublequote?re_doublequote.search(command,i)
-            :re_noquote.search(command,i))
+        i = (issinglequote?re_singlequote.indexIn(command,i)
+            :isdoublequote?re_doublequote.indexIn(command,i)
+            :re_noquote.indexIn(command,i))
     )
     {
         if ((command[i]=='(') || (command[i]=='{')) // (...) or {...}
@@ -221,9 +224,9 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
             // Replace all `...` with safer $(...)
             command.replace (i, 1, "$(");
             QRegExp re_backticks("(`|\\\\`|\\\\\\\\|\\\\\\$)");
-            for (	int i2=re_backticks.search(command,i+2);
+            for (	int i2=re_backticks.indexIn(command,i+2);
                 i2!=-1;
-                i2=re_backticks.search(command,i2)
+                i2=re_backticks.indexIn(command,i2)
             )
             {
                 if (command[i2] == '`')
@@ -267,7 +270,7 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
             if (isdoublequote)
                 v='"'+v+'"';
             else if (issinglequote)
-                v="'"+v+"'";
+                v='\''+v+'\'';
 
             command.replace (i, match.length(), v);
             i+=v.length();
@@ -284,20 +287,20 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
     }
 
     // 3. create a new process
-    kdDebug() << "CommandProc::synth: running command: " << command << endl;
-    m_commandProc = new KProcess;
+    kDebug() << "CommandProc::synth: running command: " << command;
+    m_commandProc = new K3Process;
     m_commandProc->setUseShell(true);
-    m_commandProc->setEnvironment("LANG", language + "." + codec->mimeName());
-    m_commandProc->setEnvironment("LC_CTYPE", language + "." + codec->mimeName());
+    m_commandProc->setEnvironment("LANG", language + '.' + codec->name());
+    m_commandProc->setEnvironment("LC_CTYPE", language + '.' + codec->name());
     *m_commandProc << command;
-    connect(m_commandProc, SIGNAL(processExited(KProcess*)),
-        this, SLOT(slotProcessExited(KProcess*)));
-    connect(m_commandProc, SIGNAL(receivedStdout(KProcess*, char*, int)),
-        this, SLOT(slotReceivedStdout(KProcess*, char*, int)));
-    connect(m_commandProc, SIGNAL(receivedStderr(KProcess*, char*, int)),
-        this, SLOT(slotReceivedStderr(KProcess*, char*, int)));
-    connect(m_commandProc, SIGNAL(wroteStdin(KProcess*)),
-        this, SLOT(slotWroteStdin(KProcess* )));
+    connect(m_commandProc, SIGNAL(processExited(K3Process*)),
+        this, SLOT(slotProcessExited(K3Process*)));
+    connect(m_commandProc, SIGNAL(receivedStdout(K3Process*, char*, int)),
+        this, SLOT(slotReceivedStdout(K3Process*, char*, int)));
+    connect(m_commandProc, SIGNAL(receivedStderr(K3Process*, char*, int)),
+        this, SLOT(slotReceivedStderr(K3Process*, char*, int)));
+    connect(m_commandProc, SIGNAL(wroteStdin(K3Process*)),
+        this, SLOT(slotWroteStdin(K3Process* )));
 
     // 4. start the process
 
@@ -309,14 +312,14 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
         m_state = psSynthing;
     }
     if (stdIn) {
-        m_commandProc->start(KProcess::NotifyOnExit, KProcess::All);
-        if (encodedText.length() > 0)
-            m_commandProc->writeStdin(encodedText, encodedText.length());
+        m_commandProc->start(K3Process::NotifyOnExit, K3Process::All);
+        if (encText.size() > 0)
+            m_commandProc->writeStdin(encText, encText.size());
         else
             m_commandProc->closeStdin();
     }
     else
-        m_commandProc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
+        m_commandProc->start(K3Process::NotifyOnExit, K3Process::AllOutput);
 }
 
 /**
@@ -328,7 +331,7 @@ void CommandProc::synth(const QString& inputText, const QString& suggestedFilena
 */
 QString CommandProc::getFilename()
 {
-    kdDebug() << "CommandProc::getFilename: returning " << m_synthFilename << endl;
+    kDebug() << "CommandProc::getFilename: returning " << m_synthFilename;
     return m_synthFilename;
 }
 
@@ -348,22 +351,22 @@ QString CommandProc::getFilename()
 * operation.
 */
 void CommandProc::stopText(){
-    kdDebug() << "CommandProc::stopText: Running" << endl;
+    kDebug() << "CommandProc::stopText: Running";
     if (m_commandProc)
     {
         if (m_commandProc->isRunning())
         {
-            kdDebug() << "CommandProc::stopText: killing Command." << endl;
+            kDebug() << "CommandProc::stopText: killing Command.";
             m_waitingStop = true;
             m_commandProc->kill();
         } else m_state = psIdle;
     }else m_state = psIdle;
-    kdDebug() << "CommandProc::stopText: Command stopped." << endl;
+    kDebug() << "CommandProc::stopText: Command stopped.";
 }
 
-void CommandProc::slotProcessExited(KProcess*)
+void CommandProc::slotProcessExited(K3Process*)
 {
-    kdDebug() << "CommandProc:slotProcessExited: Command process has exited." << endl;
+    kDebug() << "CommandProc:slotProcessExited: Command process has exited.";
     pluginState prevState = m_state;
     if (m_waitingStop)
     {
@@ -380,21 +383,21 @@ void CommandProc::slotProcessExited(KProcess*)
     }
 }
 
-void CommandProc::slotReceivedStdout(KProcess*, char* buffer, int buflen)
+void CommandProc::slotReceivedStdout(K3Process*, char* buffer, int buflen)
 {
     QString buf = QString::fromLatin1(buffer, buflen);
-    kdDebug() << "CommandProc::slotReceivedStdout: Received output from Command: " << buf << endl;
+    kDebug() << "CommandProc::slotReceivedStdout: Received output from Command: " << buf;
 }
 
-void CommandProc::slotReceivedStderr(KProcess*, char* buffer, int buflen)
+void CommandProc::slotReceivedStderr(K3Process*, char* buffer, int buflen)
 {
     QString buf = QString::fromLatin1(buffer, buflen);
-    kdDebug() << "CommandProc::slotReceivedStderr: Received error from Command: " << buf << endl;
+    kDebug() << "CommandProc::slotReceivedStderr: Received error from Command: " << buf;
 }
 
-void CommandProc::slotWroteStdin(KProcess*)
+void CommandProc::slotWroteStdin(K3Process*)
 {
-    kdDebug() << "CommandProc::slotWroteStdin: closing Stdin" << endl;
+    kDebug() << "CommandProc::slotWroteStdin: closing Stdin";
     m_commandProc->closeStdin();
 }
 
@@ -420,9 +423,9 @@ void CommandProc::ackFinished()
     if (m_state == psFinished)
     {
         m_state = psIdle;
-        m_synthFilename = QString::null;
+        m_synthFilename.clear();
         if (!m_textFilename.isNull()) QFile::remove(m_textFilename);
-        m_textFilename = QString::null;
+        m_textFilename.clear();
     }
 }
 
